#pragma once

#include "libsyntax.h"
#include <concepts>

namespace lib_composit {
	template<typename... TYPES> class __types_list;

	template<> class __types_list<> {
	public:
		static QList<std::shared_ptr<const lib_syntax::IBasicRule>> getRules() {
			return QList<std::shared_ptr<const lib_syntax::IBasicRule>>();
		}
	};

	template<typename T, typename... Rets>
		requires std::derived_from<T, lib_syntax::IBasicRule>
	class __types_list<T, Rets...> : public __types_list<Rets...> {
	public:
		static QList<std::shared_ptr<const lib_syntax::IBasicRule>> getRules() {
			auto list = __types_list<Rets...>::getRules();
			list.prepend(std::make_shared<const T>());
			return list;
		}
	};

	template<typename... TYPES>
	class Any : public lib_syntax::__anyone_impl, public __types_list<TYPES...> {
	public:
		Any() : __anyone_impl(__types_list<TYPES...>::getRules()) { }
	};

	template<typename... TYPES>
	class Seqs : public lib_syntax::__sequence_impl, public __types_list<TYPES...> {
	public:
		Seqs() : __sequence_impl(__types_list<TYPES...>::getRules()) { }
	};

	template<typename TYPE, int min, int max> requires std::derived_from<TYPE, lib_syntax::IBasicRule>
	class Rept : public lib_syntax::__repeat_impl {
	public:
		Rept() : __repeat_impl(std::make_shared<TYPE>(), min, max) { }
	};
	template<typename TYPE> class OptMulti : public Rept<TYPE, 0, INT_MAX> { };
	template<typename TYPE> class Multi : public Rept<TYPE, 1, INT_MAX> { };
	template<typename TYPE> class Opt : public Rept<TYPE, 0, 1> { };


	template<typename E, typename T, lib_token::TokenProc<E> p = nullptr>
		requires std::derived_from<E, ast_basic::IExprInstance>&& std::derived_from<T, lib_token::ITokenProcess>
	class Action : public lib_syntax::__token_match_impl<E, p> {
	public:
		Action() : __token_match_impl<E, p>(std::make_shared<T>()) { }
	};

	template<typename ExprT> requires std::derived_from<ExprT, ast_basic::IExprInstance>
	void apntk(std::shared_ptr<ExprT> expr, std::shared_ptr<const lib_token::IToken> t) {
		expr->addToken(t);
	}
	template<typename T>
		requires std::derived_from<T, lib_token::ITokenProcess>
	class Match : public Action<ast_basic::IExprInstance, T, apntk<ast_basic::IExprInstance>> { };
}
